@@ -7,8 +7,6 @@ st_as_sfc.stars = function(x, ..., as_points, which = seq_len(prod(dim(x)[1:2]))
 	r = attr(st_dimensions(x), "raster")
 	gt = get_geotransform(x)
 	d = st_dimensions(x)[r$dimensions]
-	if (utils::packageVersion("sf") <= "0.7-2" && is_rectilinear(x))
-		stop("for converting rectilinear grids to sf, upgrade sf to a version > 0.7-2")
 	st_as_sfc(d, ..., as_points = as_points, which = which, geotransform = gt) 
 }
 
@@ -78,7 +76,7 @@ st_xy2sfc = function(x, as_points, ..., na.rm = TRUE) {
 #' @param merge logical; if \code{TRUE}, cells with identical values are merged (using \code{GDAL_Polygonize} or \code{GDAL_FPolygonize}); if \code{FALSE}, a polygon for each raster cell is returned; see details
 #' @param use_integer (relevant only if \code{merge} is \code{TRUE}): if \code{TRUE}, before polygonizing values are rounded to 32-bits signed integer values (GDALPolygonize), otherwise they are converted to 32-bit floating point values (GDALFPolygonize).
 #' @param long logical; if \code{TRUE}, return a long table form \code{sf}, with geometries and other dimensinos recycled
-#' @param connect8 logical; if \code{TRUE}, use 8 connectedness. Otherwise the 4 connectedness algorithm will be applied (this option can only be made to work after sf 0.7-3 has arrived on CRAN).
+#' @param connect8 logical; if \code{TRUE}, use 8 connectedness. Otherwise the 4 connectedness algorithm will be applied.
 #' @param ... ignored
 #' @details If \code{merge} is \code{TRUE}, only the first attribute is converted into an \code{sf} object. If \code{na.rm} is \code{FALSE}, areas with \code{NA} values are also written out as polygons. Note that the resulting polygons are typically invalid, and use \link[lwgeom]{st_make_valid} to create valid polygons out of them.
 #' @export
@@ -107,15 +105,8 @@ st_as_sf.stars = function(x, ..., as_points = FALSE, merge = FALSE, na.rm = TRUE
 			} else
 				NULL
 
-		if (connect8)
-			stop("connect8 will work after sf 0.7-3 is on CRAN")
-
 		ret = gdal_polygonize(x, mask, use_integer = use_integer, geotransform = get_geotransform(x),
-				use_contours = FALSE, ...)
-
-#			else # FIXME: make this work after sf 0.7-3 hits CRAN
-#				gdal_polygonize(x, mask, use_integer = use_integer, geotransform = get_geotransform(x),
-#					use_contours = FALSE, connect8 = connect8)
+				use_contours = FALSE, connect8 = connect8, ...)
 
 		# factor levels?
 		if (!is.null(lev <- attr(x[[1]], "levels")))
